// Generated by CoffeeScript 1.10.0
(function() {
  var Parser, ParserOptions, _, acorn, acornjsx, ast, async, bna, fs, jsParse, log, path, resolver, util, wrench,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  fs = require("fs");

  resolver = require("resolve");

  async = require("async");

  path = require("path");

  _ = require("underscore");

  ast = require("./ast");

  wrench = require("wrench");

  util = require('util');

  log = require("lawg");

  require("colors");

  acornjsx = require("acorn-jsx");

  acorn = require("acorn");

  Parser = acorn;

  ParserOptions = {
    ecmaVersion: 6,
    allowImportExportEverywhere: true,
    allowHashBang: true,
    allowReturnOutsideFunction: true,
    locations: false
  };

  jsParse = (function(_this) {
    return function(src) {
      return Parser.parse(src, _.extend({}, ParserOptions, {
        locations: bna.locations
      }));
    };
  })(this);

  module.exports = bna = {
    enableJsx: function() {
      Parser = acornjsx;
      return ParserOptions.plugins = {
        jsx: true
      };
    },
    quiet: false,
    locations: false,
    warn: function(msg) {
      if (!bna.quiet) {
        return log(msg.gray);
      }
    },
    _cache: {},

    /*
     * given path to js file (as returned by require.resolve), identify the node module that contains it, think of it
     * as reverse of require.resolve
     * returns
     *
     * @param file  : path to js file to identify
     * @returns  {
     *     "mname": "mylib",    # module name, as used by require
     *     "path"   : "node_modules/mylib/file.js",
     *     "mpath"  : "node_modules/mylib",             # the path of module that contains this file
     *     "package" : { package.json object if exists }
     *   }
     *   - "mpath" may point to a file in case of node_modules/mylib.js
     */
    identify: function(file) {
      var packageJsonFile, ret;
      ret = {};
      ret.mpath = /\/|\\/.test(file) ? bna.findModulePath(file) : file;
      if (ret.mpath === file) {
        ret.mname = path.basename(ret.mpath).replace(/\.(js|node)$/i, '');
      } else {
        ret.mname = path.basename(ret.mpath);
        packageJsonFile = path.join(ret.mpath, "package.json");
        if (fs.existsSync(packageJsonFile)) {
          ret["package"] = JSON.parse(fs.readFileSync(packageJsonFile));
        }
      }
      return ret;
    },
    findModulePath: function(fullpath) {
      var packageJsonFile, parentpath;
      if (!fullpath || fullpath === "/") {
        return void 0;
      }
      if (fs.existsSync(path.join(fullpath, "package.json"))) {
        return fullpath;
      }
      parentpath = path.dirname(fullpath);
      packageJsonFile = path.join(parentpath, "package.json");
      if (fs.existsSync(packageJsonFile)) {
        return parentpath;
      } else if (path.basename(parentpath) === "node_modules") {
        return fullpath;
      } else {
        return bna.findModulePath(parentpath);
      }
    },
    mainFile: function(mpath) {
      fpath;
      var fpath, pkg;
      if (fs.existsSync(fpath = path.join(mpath, "package.json"))) {
        pkg = JSON.parse(fs.readFileSync(fpath));
        if ("main" in pkg) {
          return path.resolve(mpath, pkg.main);
        }
      } else if (fs.existsSync(fpath = path.join(mpath, "index.js"))) {
        return fpath;
      } else if (fs.existsSync(fpath = path.join(mpath, "index.node"))) {
        return fpath;
      }
      return null;
    },

    /*
      Internal, collapse dependent files belong to same package
      assumes unit.requires is [ {node, unit} ....] when passed in (as returned by _parseFile)
      on output, unit.requires is [ unit, .... ],  the other non consequential members are stripped out
      the recursive tree data structure is preserved.
     */
    _collapsePackages: function(unit) {
      var doCollapse, memory;
      memory = {};
      doCollapse = function(unit) {
        var detail, reqs;
        memory[unit.mpath] = true;
        detail = _({}).extend(unit);
        if (!detail.requires) {
          return detail;
        }
        reqs = _(detail.requires).reduce(function(memo, arg1) {
          var unit;
          unit = arg1.unit;
          if (detail.mpath === unit.mpath) {
            if (unit.requires !== void 0) {
              memo = _(memo).concat((function() {
                var j, len, ref, results;
                ref = unit.requires;
                results = [];
                for (j = 0, len = ref.length; j < len; j++) {
                  unit = ref[j].unit;
                  results.push(unit);
                }
                return results;
              })());
            }
          } else {
            memo.push(unit);
          }
          return memo;
        }, []);
        detail.requires = _(reqs).chain().map(function(unit) {
          if (memory[unit.mpath]) {
            return unit;
          } else {
            return doCollapse(unit);
          }
        }).unique(function(unit) {
          return unit.mpath;
        }).value();
        return detail;
      };
      return doCollapse(unit);
    },

    /*
     * figures out module dependencies for you,
     *  how does it work?  by finding the main file for a package, then walk through ast of the file, find all requires
     *  to determine the external&local packages in its dependencies.
     *   * external : a required external and non-system node module
     *   * local    : the module exists locally (can be resolved via require.resolve)
     *
     * @param fpath          : path of main module file, should be what's returned by require.resolve('module')
     * @param cb(err, alldpes, externdeps, main)
     *  alldeps:  all dependencies, an object, value could be:
                  - string,  single version
                  - array of string, multiple versions
                  - null, key is either a built-in module, or a full path file (not a module main file)
        externdeps:  dependences that do not reside in fpath
        main :       the main detail for fpath
        warnings :   require resolve warnings
     */
    npmDependencies: function(fpath, cb) {
      var dependencies, e, error1, externDeps, unit, warnings;
      try {
        unit = bna._parseFile(fpath, bna._cache);
        warnings = _((function() {
          var j, len, ref, results;
          ref = bna._flattenRequires(unit);
          results = [];
          for (j = 0, len = ref.length; j < len; j++) {
            unit = ref[j];
            results.push(unit.warnings);
          }
          return results;
        })()).flatten();
        unit = bna._collapsePackages(unit);
        dependencies = null;
        externDeps = null;
        dependencies = _(unit.requires).reduce(function(memo, unit) {
          if (unit["package"]) {
            memo[unit.mname] = unit["package"].version;
          } else {
            if (unit.isCore) {
              memo[unit.fpath] = null;
            } else {
              memo[unit.fpath] = null;
            }
            memo;
          }
          return memo;
        }, {});
        externDeps = bna.externDeps(unit);
        return cb(null, dependencies, externDeps, unit, warnings);
      } catch (error1) {
        e = error1;
        return cb(e);
      }
    },

    /*
     *  Give a module, find all its dependencies that are NOT located in its local node_module path, useful
     *  for building the final app.
    
     *  Internal helper called by npmDependencies.
     *
     */
    externDeps: function(unit) {
      var isPathContained, memory, ret, walk;
      ret = [];
      isPathContained = function(path) {
        var e, i, j, len;
        for (i = j = 0, len = ret.length; j < len; i = ++j) {
          e = ret[i];
          if (path.indexOf(e.mpath) === 0) {
            return true;
          }
        }
        return false;
      };
      memory = {};
      walk = function(unit) {
        memory[unit.mpath] = true;
        if (!unit.isCore && !isPathContained(unit.mpath)) {
          ret.push(unit);
        }
        if (unit.requires) {
          return unit.requires.forEach(function(unit) {
            if (!memory[unit.mpath]) {
              return walk(unit);
            }
          });
        }
      };
      walk(unit);
      ret = _(ret).chain().map(function(unit) {
        return {
          'require': unit.mname,
          'mpath': unit.mpath,
          'version': unit["package"] ? unit["package"].version : null
        };
      }).unique(function(unit) {
        return unit.mpath;
      }).value();
      return ret;
    },
    dir: {
      _scanDir: function(dir, iteratorCb, doneCb) {
        return async.waterfall([
          function(cb) {
            var files, fnpm;
            files = [];
            fnpm = require("fstream-npm");
            return fnpm({
              path: dir
            }).on("child", function(c) {
              return files.push(c._path);
            }).on('close', function() {
              return cb(null, files);
            });
          }, function(files, cb) {
            return async.each(files, function(file, cb) {
              return fs.stat(file, function(err, stat) {
                if (stat.isDirectory() && path.basename(file) !== 'node_modules') {
                  return iteratorCb(file, true, cb);
                } else if (path.extname(file) === ".js") {
                  return iteratorCb(file, false, cb);
                } else {
                  return cb();
                }
              });
            }, cb);
          }
        ], doneCb);
      },
      npmDependencies: function(dir, cb) {
        var alldeps, allextdeps, allwarnings;
        alldeps = {};
        allextdeps = [];
        allwarnings = [];
        return bna.dir._scanDir(dir, function(file, isDir, cb) {
          var f;
          f = isDir ? bna.dir.npmDependencies : bna.npmDependencies;
          return f(file, function(err, deps, extdeps, main, warnings) {
            _(alldeps).extend(deps);
            allextdeps = _(allextdeps).concat(extdeps);
            allwarnings = _(allwarnings).concat(warnings);
            return cb(err);
          });
        }, function(err) {
          var detail;
          detail = bna.identify(dir);
          delete alldeps[detail.mname];
          allextdeps = _(allextdeps).chain().unique(function(d) {
            return d.mpath;
          }).filter(function(d) {
            return d.mpath.indexOf(dir) !== 0;
          }).value();
          return cb(err, alldeps, allextdeps, {
            mpath: dir
          }, allwarnings);
        });
      }
    },

    /*
     * Make extern dependencies local by copying them to local node_modules folder
     * @param mpath     path to a dir or file (the file must be main entry point of module, i.e. returned by require.resolve
     * @param progressCb (msg),  copy in progress callback, if you want to print status
     * @param doneCb(err) completion callback
     *
     */
    copyExternDependModules: function(mpath, progressCb, doneCb) {
      return fs.stat(mpath, function(err, stat) {
        var f;
        if (err) {
          return cb(err);
        }
        f = stat.isDirectory() ? bna.dir.npmDependencies : bna.npmDependencies;
        return f(mpath, function(err, __d, extDependencies, main) {
          var targetPath;
          targetPath = path.join(main.mpath, "node_modules");
          if (!fs.existsSync(targetPath)) {
            wrench.mkdirSyncRecursive(targetPath);
          }
          return async.eachSeries(extDependencies, function(d, cb) {
            var targetModulePath;
            targetModulePath = path.join(targetPath, path.basename(d.mpath));
            progressCb(util.format("Copying '%s': %s => %s", d.mname, path.relative(process.cwd(), d.mpath), path.relative(process.cwd(), targetModulePath)));
            if (!fs.existsSync(targetModulePath)) {
              fs.mkdirSync(targetModulePath);
            }
            return wrench.copyDirRecursive(d.mpath, targetModulePath, cb);
          }, doneCb);
        });
      });
    },

    /*
     * Merge the module dependency calculated by bna.npmDependencies or bna.dir.npmDependencies into the package.json
     * of main module. The merge rules are:
     * 1. if not exist yet, dependency is added
     * 2. otherwise, do not modify current package.json
          a. however if detected dependency exits in current package.json, then check if versions are compatible,
     *
     * @param mpath     path to a dir or file (the file must be main entry point of module, i.e. returned by require.resolve
     * @param cb(err, [] )  [] is removed packages
     */
    writePackageJson: function(mpath, cb) {
      var semver;
      semver = require("semver");
      return fs.stat(mpath, function(err, stat) {
        var f;
        if (err) {
          return cb(err);
        }
        f = stat.isDirectory() ? bna.dir.npmDependencies : bna.npmDependencies;
        return f(mpath, function(err, deps) {
          var errList, newdep, oldDep, pkgJson, pkgJsonFile;
          if (err) {
            return cb(err);
          }
          if (stat.isFile()) {
            mpath = bna.identify(mpath).mpath;
          }
          pkgJsonFile = path.join(mpath, "package.json");
          pkgJson = {};
          if (fs.existsSync(pkgJsonFile)) {
            pkgJson = JSON.parse(fs.readFileSync(pkgJsonFile, "utf8"));
          }
          oldDep = pkgJson.dependencies || {};
          newdep = {};
          errList = [];
          _(deps).each(function(version, name) {
            var oldVer;
            if (version === null) {

            } else if (!(name in oldDep)) {
              return newdep[name] = version;
            } else {
              oldVer = oldDep[name];
              if (/:\/\//.test(oldVer)) {
                log(util.format("Package %s is ignored due to non-semver %s", name, oldVer));
                delete oldDep[name];
                return newdep[name] = oldVer;
              } else if (!semver.satisfies(version, oldVer)) {
                return errList.push(util.format("%s: actual version %s does not satisfy package.json's version %s", name, version, oldVer));
              } else {
                delete oldDep[name];
                return newdep[name] = oldVer;
              }
            }
          });
          if (errList.length > 0) {
            return cb(new Error(errList.join("\n")));
          } else {
            pkgJson.dependencies = newdep;
            return fs.writeFile(pkgJsonFile, JSON.stringify(pkgJson, null, 2), "utf8", function(err) {
              return cb(err, _(oldDep).keys());
            });
          }
        });
      });
    },
    _flattenRequires: function(unit) {
      var getreqs, units;
      getreqs = function(unit, cache) {
        var child_unit, d, deps, j, len, node, units;
        if (unit.fpath in cache) {
          return cache[unit.fpath];
        }
        cache[unit.fpath] = units = [];
        deps = _((function() {
          var j, len, ref, ref1, results;
          ref = unit.requires;
          results = [];
          for (j = 0, len = ref.length; j < len; j++) {
            ref1 = ref[j], node = ref1.node, child_unit = ref1.unit;
            results.push(getreqs(child_unit, cache));
          }
          return results;
        })()).flatten();
        for (j = 0, len = deps.length; j < len; j++) {
          d = deps[j];
          units.push(d);
        }
        units.push(unit);
        return units;
      };
      units = getreqs(unit, {});
      return _(units).unique(function(unit) {
        return unit.fpath;
      });
    },
    _fuse: function(filepath, outdir, moduleName, opts) {
      var ret, unit, units, warnings;
      if (opts == null) {
        opts = {};
      }
      filepath = path.resolve(filepath);
      unit = bna._parseFile(filepath, {}, true, opts.fakeCode);
      if (!outdir) {
        outdir = path.dirname(filepath);
      }
      units = bna._flattenRequires(unit);
      warnings = _((function() {
        var j, len, results;
        results = [];
        for (j = 0, len = units.length; j < len; j++) {
          unit = units[j];
          results.push(unit.warnings);
        }
        return results;
      })()).flatten();
      ret = (require("./fuse")).generate({
        baseDir: outdir,
        moduleName: moduleName,
        units: units,
        asLib: opts.aslib,
        generateSm: opts.generateSm,
        prependCode: opts.prependCode || ''
      });
      ret.push.apply(ret, [warnings, units]);
      return ret;
    },
    generateModuleName: function(fullpath) {
      var ret;
      ret = path.basename(fullpath).replace(/\..*$/, '');
      if (ret.toLowerCase() === "index") {
        ret = path.basename(path.dirname(fullpath));
      }
      return ret;
    },
    prettyWarnings: function(warnings) {
      var error, lines, locs, modules, msgs, node, pe, reason, warnings1, warnings2;
      msgs = {
        'nonconst': 'require dynamic modules: ',
        'resolve': 'require ignored because parameter can not be resolved',
        'dynamicResolveError': 'dynamic required module resolve error'
      };
      pe = function(e) {
        if (e) {
          return ', ' + e;
        } else {
          return '';
        }
      };
      warnings1 = (function() {
        var j, len, ref, results;
        results = [];
        for (j = 0, len = warnings.length; j < len; j++) {
          ref = warnings[j], node = ref.node, reason = ref.reason, error = ref.error;
          if (reason !== 'nonconst') {
            results.push((path.relative('.', node.loc.file)) + ":" + node.loc.line + ": " + msgs[reason] + (pe(error)));
          }
        }
        return results;
      })();
      lines = (function(_this) {
        return function(locs) {
          var l;
          return ((function() {
            var j, len, results;
            results = [];
            for (j = 0, len = locs.length; j < len; j++) {
              l = locs[j];
              results.push(l.line);
            }
            return results;
          })()).join(',');
        };
      })(this);
      warnings2 = (function() {
        var j, len, ref, results;
        results = [];
        for (j = 0, len = warnings.length; j < len; j++) {
          ref = warnings[j], reason = ref.reason, locs = ref.locs, modules = ref.modules;
          if (reason === 'nonconst') {
            results.push((path.relative('.', locs[0].file)) + ":" + (lines(locs)) + ": " + msgs[reason] + (modules.join(',')));
          }
        }
        return results;
      })();
      return warnings1.concat(warnings2);
    },
    fuseTo: function(filepath, dstdir, opts) {
      var binaryunits, bunit, content, dfile, dstfile, j, k, len, len1, moduleName, ref, ref1, smFile, sourcemap, units, warning, warnings;
      if (opts == null) {
        opts = {};
      }
      filepath = path.resolve(filepath);
      moduleName = bna.generateModuleName(opts.dstfile || filepath);
      ref = bna._fuse(filepath, dstdir, moduleName, opts), content = ref[0], binaryunits = ref[1], sourcemap = ref[2], warnings = ref[3], units = ref[4];
      ref1 = bna.prettyWarnings(warnings);
      for (j = 0, len = ref1.length; j < len; j++) {
        warning = ref1[j];
        bna.warn(warning);
      }
      wrench.mkdirSyncRecursive(dstdir);
      dstfile = path.resolve(dstdir, opts.dstfile || (path.basename(filepath, ".js") + ".fused.js"));
      smFile = dstfile + ".map";
      log("Generating " + (path.relative('.', dstfile)));
      fs.writeFileSync(dstfile, content);
      if (sourcemap) {
        log("Generating " + (path.relative('.', smFile)));
        sourcemap.file = path.basename(dstfile);
        fs.writeFileSync(smFile, JSON.stringify(sourcemap, null, 2));
        fs.appendFileSync(dstfile, "\n//# sourceMappingURL=" + (path.basename(smFile)));
      }
      for (k = 0, len1 = binaryunits.length; k < len1; k++) {
        bunit = binaryunits[k];
        dfile = path.resolve(dstdir, bunit.binName);
        if (fs.existsSync(dfile)) {
          bna.warn("Skipped copying " + dfile + ", already exists.");
        } else {
          wrench.mkdirSyncRecursive(path.dirname(dfile));
          fs.createReadStream(bunit.fpath).pipe(fs.createWriteStream(dfile));
        }
      }
      return units;
    },
    fuseDirTo: function(dirpath, dstdir, opts, cb) {
      var fakeCode, scandir;
      fakeCode = "";
      scandir = function(curdir, cb) {
        var files, fnpm;
        files = [];
        fnpm = require("fstream-npm");
        return fnpm({
          path: curdir
        }).on("child", function(c) {
          return files.push(c._path);
        }).on('close', function() {
          var extname, fpath, j, len, mpath, name, pkg, recursePaths, stat;
          recursePaths = [];
          for (j = 0, len = files.length; j < len; j++) {
            name = files[j];
            fpath = path.resolve(curdir, name);
            stat = fs.statSync(fpath);
            if (stat.isFile()) {
              extname = path.extname(name).toLowerCase();
              if (name.toLowerCase() === "package.json") {
                pkg = JSON.parse(fs.readFileSync(fpath));
                if (pkg.main) {
                  mpath = path.resolve(path.dirname(fpath), pkg.main);
                  fakeCode += "require('./" + (path.relative(dirpath, mpath)) + "')\n";
                }
              } else if (extname === ".js" || extname === ".node") {
                fakeCode += "require('./" + (path.relative(dirpath, fpath)) + "')\n";
              }
            } else if (stat.isDirectory()) {
              if (fs.existsSync(path.resolve(fpath, "package.json")) || fs.existsSync(path.resolve(fpath, "index.js")) || fs.existsSync(path.resolve(fpath, "index.node"))) {
                fakeCode += "require('./" + (path.relative(dirpath, fpath)) + "')\n";
              } else {
                recursePaths.push(fpath);
              }
            }
          }
          return async.eachSeries(recursePaths, function(path, cb) {
            return scandir(fpath, cb);
          }, cb);
        });
      };
      return scandir(dirpath, function() {
        var units;
        if (!fakeCode) {
          log("No files detected");
        } else {
          opts.fakeCode = fakeCode;
          units = bna.fuseTo(path.resolve(dirpath, "lib.js"), dstdir, opts);
        }
        if (cb) {
          return cb(units);
        }
      });
    },
    fileDep: function(filepath) {
      var getreqs, j, len, ret, u, unit, units, warnings;
      filepath = path.resolve(filepath);
      unit = bna._parseFile(filepath, {});
      getreqs = function(unit, cache) {
        var child_unit, node, units;
        if (unit.fpath in cache) {
          return cache[unit.fpath];
        }
        cache[unit.fpath] = units = [];
        units = _(units).concat(_((function() {
          var j, len, ref, ref1, results;
          ref = unit.requires;
          results = [];
          for (j = 0, len = ref.length; j < len; j++) {
            ref1 = ref[j], node = ref1.node, child_unit = ref1.unit;
            results.push(getreqs(child_unit, cache));
          }
          return results;
        })()).flatten());
        units.push(unit);
        return units;
      };
      units = getreqs(unit, {});
      units = _(units).unique(function(unit) {
        return unit.fpath;
      });
      warnings = _((function() {
        var j, len, results;
        results = [];
        for (j = 0, len = units.length; j < len; j++) {
          unit = units[j];
          results.push(unit.warnings);
        }
        return results;
      })()).flatten();
      ret = {};
      for (j = 0, len = units.length; j < len; j++) {
        u = units[j];
        if (u["package"]) {
          ret[u["package"].name] = u["package"].version;
        }
      }
      return [ret, warnings];
    },

    /*
     * helper: parse source code, recursively analyze require in code, return results in a nested tree structure
     * filepath: must be absolute path
     */
    _parseFile: function(filepath, cache, ifStoreSrc, fakeCode) {
      var code, dynLocs, e, error1, isBinary, isCore, src, unit;
      if (filepath in cache) {
        return cache[filepath];
      }
      isCore = !/[\\\/]/.test(filepath);
      isBinary = /\.node$/i.test(filepath);
      unit = {
        isCore: isCore,
        isBinary: isBinary,
        fpath: filepath,
        mpath: '',
        mname: '',
        src: "",
        requires: [],
        warnings: [],
        "package": void 0
      };
      cache[filepath] = unit;
      if (isCore || isBinary) {
        return unit;
      }
      if (path.extname(filepath).toLowerCase() === ".json") {
        if (ifStoreSrc) {
          unit.src = fs.readFileSync(filepath).toString();
        }
        return unit;
      }
      (function() {
        var detail, main, mainfiles, ref;
        detail = bna.identify(filepath);
        unit.mname = detail.mname;
        unit.mpath = detail.mpath;
        unit._detailPackage = detail["package"];
        mainfiles = [path.join(detail.mpath, "index.js"), path.join(detail.mpath, "index.node")];
        if (detail["package"]) {
          detail["package"].name = detail.mname;
          if (detail["package"].main) {
            main = path.resolve(detail.mpath, detail["package"].main);
            mainfiles.push(main);
            if (path.extname(main) === '') {
              mainfiles.push.apply(mainfiles, [main + ".js", main + ".node"]);
            }
          }
        }
        if (ref = unit.fpath, indexOf.call(mainfiles, ref) >= 0) {
          return unit["package"] = detail["package"] || {
            name: detail.mname,
            version: 'x'
          };
        }
      })();
      try {
        if (fakeCode) {
          src = fakeCode;
        } else {
          src = fs.readFileSync(filepath).toString().replace(/^#![^\n]*\n/, '');
        }
        if (ifStoreSrc) {
          unit.src = src;
        }
        code = jsParse(src);
      } catch (error1) {
        e = error1;
        log("Ignoring " + filepath + ", failed to parse due to: " + e);
        return unit;
      }
      dynLocs = [];
      ast.traverse(code, [ast.isRequire], function(node) {
        var arg, error2, fullpath, modulename, runit;
        if (!node.loc) {
          node.loc = {
            file: filepath,
            line: '?'
          };
        } else {
          node.loc.file = filepath;
          node.loc.line = node.loc.start.line;
        }
        arg = node["arguments"][0];
        if (arg && arg.type === 'Literal') {
          modulename = arg.value;
          e = void 0;
          try {
            fullpath = resolver.sync(modulename, {
              extensions: ['.js', '.node', '.json'],
              basedir: path.dirname(filepath)
            });
          } catch (error2) {
            e = error2;
            unit.warnings.push({
              node: node,
              reason: "resolve"
            });
          }
          if (!e) {
            runit = bna._parseFile(fullpath, cache, ifStoreSrc);
            return unit.requires.push({
              name: modulename,
              node: node,
              unit: runit
            });
          }
        } else {
          return dynLocs.push(node.loc);
        }
      });
      if (dynLocs.length > 0) {
        (function(_this) {
          return (function() {
            var dynamicModules, error2, fullpath, j, len, m, modulename, node, runit;
            dynamicModules = bna.detectDynamicRequires(unit);
            dynamicModules = (function() {
              var j, len, results;
              results = [];
              for (j = 0, len = dynamicModules.length; j < len; j++) {
                m = dynamicModules[j];
                if (m && !_(unit.requires).find((function(_this) {
                  return function(e) {
                    return e.name === m;
                  };
                })(this))) {
                  results.push(m);
                }
              }
              return results;
            }).call(_this);
            for (j = 0, len = dynamicModules.length; j < len; j++) {
              modulename = dynamicModules[j];
              e = void 0;
              fullpath = '';
              try {
                fullpath = resolver.sync(modulename, {
                  extensions: ['.js', '.node', '.json'],
                  basedir: path.dirname(filepath)
                });
                node = {
                  loc: {
                    file: fullpath,
                    line: '?'
                  }
                };
                runit = bna._parseFile(fullpath, cache, ifStoreSrc);
                unit.requires.push({
                  name: modulename,
                  node: node,
                  unit: runit
                });
              } catch (error2) {
                e = error2;
                unit.warnings.push({
                  node: {
                    loc: {
                      file: filepath,
                      line: '?'
                    }
                  },
                  reason: "dynamicResolveError",
                  error: e
                });
              }
            }
            return unit.warnings.push({
              locs: dynLocs,
              modules: dynamicModules,
              reason: "nonconst"
            });
          });
        })(this)();
      }
      return unit;
    },
    detectDynamicRequires: function(unit) {
      var _r, dmodules, e, error1, src, tmpfile;
      src = "var _sysreq = require;\n(function() {\nvar dmodules = []\nvar require = function(module) {\n  dmodules.push(module)\n  return _sysreq(module)\n};\n" + unit.src + "\nmodule.exports = dmodules;\n})()";
      tmpfile = unit.fpath + ".bna.js";
      fs.writeFileSync(tmpfile, src);
      dmodules = [];
      try {
        _r = require;
        return dmodules = _.unique(_r(tmpfile));
      } catch (error1) {
        e = error1;
        return unit.warnings.push({
          node: {
            loc: {
              file: unit.fpath,
              line: '?'
            }
          },
          reason: 'dynamicResolveError',
          error: e
        });
      } finally {
        delete require.cache[tmpfile];
        fs.unlinkSync(tmpfile);
        return dmodules;
      }
    }
  };

}).call(this);

//# sourceMappingURL=bna.js.map
